@@ -1,11 +1,11 @@
 import os, re, weakref, struct, time
 
-from foolscap import Referenceable
+from foolscap.api import Referenceable
 from twisted.application import service
 
 from zope.interface import implements
 from allmydata.interfaces import RIStorageServer, IStatsProducer
-from allmydata.util import fileutil, log, time_format
+from allmydata.util import fileutil, idlib, log, time_format
 import allmydata # for __full_version__
 
 from allmydata.storage.common import si_b2a, si_a2b, storage_index_to_dir
@@ -36,6 +36,16 @@ class StorageServer(service.MultiService, Referenceable):
     implements(RIStorageServer, IStatsProducer)
     name = 'storage'
     LeaseCheckerClass = LeaseCheckingCrawler
+    windows = False
+
+    try:
+        import win32api, win32con
+        windows = True
+        # <http://msdn.microsoft.com/en-us/library/ms680621%28VS.85%29.aspx>
+        win32api.SetErrorMode(win32con.SEM_FAILCRITICALERRORS |
+                              win32con.SEM_NOOPENFILEERRORBOX)
+    except ImportError:
+        pass
 
     def __init__(self, storedir, nodeid, reserved_space=0,
                  discard_storage=False, readonly_storage=False,
@@ -66,11 +76,11 @@ class StorageServer(service.MultiService, Referenceable):
         self._clean_incomplete()
         fileutil.make_dirs(self.incomingdir)
         self._active_writers = weakref.WeakKeyDictionary()
-        lp = log.msg("StorageServer created", facility="tahoe.storage")
+        log.msg("StorageServer created", facility="tahoe.storage")
 
         if reserved_space:
             if self.get_available_space() is None:
-                log.msg("warning: [storage]reserved_space= is set, but this platform does not support statvfs(2), so this reservation cannot be honored",
+                log.msg("warning: [storage]reserved_space= is set, but this platform does not support an API to get disk statistics (statvfs(2) or GetDiskFreeSpaceEx), so this reservation cannot be honored",
                         umin="0wZ27w", level=log.UNUSUAL)
 
         self.latencies = {"allocate": [], # immutable
@@ -96,6 +106,9 @@ class StorageServer(service.MultiService, Referenceable):
                                    expiration_sharetypes)
         self.lease_checker.setServiceParent(self)
 
+    def __repr__(self):
+        return "<StorageServer %s>" % (idlib.shortnodeid_b2a(self.my_nodeid),)
+
     def add_bucket_counter(self):
         statefile = os.path.join(self.storedir, "bucket_counter.state")
         self.bucket_counter = BucketCountingCrawler(self, statefile)
@@ -147,22 +160,35 @@ class StorageServer(service.MultiService, Referenceable):
     def _clean_incomplete(self):
         fileutil.rm_dir(self.incomingdir)
 
-    def do_statvfs(self):
-        return os.statvfs(self.storedir)
+    def get_disk_stats(self):
+        """Return disk statistics for the storage disk, in the form of a dict
+        with the following fields.
+          total:            total bytes on disk
+          free_for_root:    bytes actually free on disk
+          free_for_nonroot: bytes free for "a non-privileged user" [Unix] or
+                              the current user [Windows]; might take into
+                              account quotas depending on platform
+          used:             bytes used on disk
+          avail:            bytes available excluding reserved space
+        An AttributeError can occur if the OS has no API to get disk information.
+        An EnvironmentError can occur if the OS call fails."""
+
+        if self.windows:
+            # For Windows systems, where os.statvfs is not available, use GetDiskFreeSpaceEx.
+            # <http://docs.activestate.com/activepython/2.5/pywin32/win32api__GetDiskFreeSpaceEx_meth.html>
+            #
+            # Although the docs say that the argument should be the root directory
+            # of a disk, GetDiskFreeSpaceEx actually accepts any path on that disk
+            # (like its Win32 equivalent).
+
+            (free_for_nonroot, total, free_for_root) = self.win32api.GetDiskFreeSpaceEx(self.storedir)
+        else:
+            # For Unix-like systems.
+            # <http://docs.python.org/library/os.html#os.statvfs>
+            # <http://opengroup.org/onlinepubs/7990989799/xsh/fstatvfs.html>
+            # <http://opengroup.org/onlinepubs/7990989799/xsh/sysstatvfs.h.html>
+            s = os.statvfs(self.storedir)
 
-    def get_stats(self):
-        # remember: RIStatsProvider requires that our return dict
-        # contains numeric values.
-        stats = { 'storage_server.allocated': self.allocated_size(), }
-        stats["storage_server.reserved_space"] = self.reserved_space
-        for category,ld in self.get_latencies().items():
-            for name,v in ld.items():
-                stats['storage_server.latencies.%s.%s' % (category, name)] = v
-        writeable = True
-        if self.readonly_storage:
-            writeable = False
-        try:
-            s = self.do_statvfs()
             # on my mac laptop:
             #  statvfs(2) is a wrapper around statfs(2).
             #    statvfs.f_frsize = statfs.f_bsize :
@@ -173,55 +199,67 @@ class StorageServer(service.MultiService, Referenceable):
             # wrong, and s.f_blocks*s.f_frsize is twice the size of my disk,
             # but s.f_bavail*s.f_frsize is correct
 
-            disk_total = s.f_frsize * s.f_blocks
-            disk_used = s.f_frsize * (s.f_blocks - s.f_bfree)
-            # spacetime predictors should look at the slope of disk_used.
-            disk_free_for_root = s.f_frsize * s.f_bfree
-            disk_free_for_nonroot = s.f_frsize * s.f_bavail
+            total = s.f_frsize * s.f_blocks
+            free_for_root = s.f_frsize * s.f_bfree
+            free_for_nonroot = s.f_frsize * s.f_bavail
 
-            # include our local policy here: if we stop accepting shares when
-            # the available space drops below 1GB, then include that fact in
-            # disk_avail.
-            disk_avail = disk_free_for_nonroot - self.reserved_space
-            disk_avail = max(disk_avail, 0)
-            if self.readonly_storage:
-                disk_avail = 0
-            if disk_avail == 0:
-                writeable = False
+        # valid for all platforms:
+        used = total - free_for_root
+        avail = max(free_for_nonroot - self.reserved_space, 0)
+
+        return { 'total': total, 'free_for_root': free_for_root,
+                 'free_for_nonroot': free_for_nonroot,
+                 'used': used, 'avail': avail, }
+
+    def get_stats(self):
+        # remember: RIStatsProvider requires that our return dict
+        # contains numeric values.
+        stats = { 'storage_server.allocated': self.allocated_size(), }
+        stats['storage_server.reserved_space'] = self.reserved_space
+        for category,ld in self.get_latencies().items():
+            for name,v in ld.items():
+                stats['storage_server.latencies.%s.%s' % (category, name)] = v
+
+        try:
+            disk = self.get_disk_stats()
+            writeable = disk['avail'] > 0
 
             # spacetime predictors should use disk_avail / (d(disk_used)/dt)
-            stats["storage_server.disk_total"] = disk_total
-            stats["storage_server.disk_used"] = disk_used
-            stats["storage_server.disk_free_for_root"] = disk_free_for_root
-            stats["storage_server.disk_free_for_nonroot"] = disk_free_for_nonroot
-            stats["storage_server.disk_avail"] = disk_avail
+            stats['storage_server.disk_total'] = disk['total']
+            stats['storage_server.disk_used'] = disk['used']
+            stats['storage_server.disk_free_for_root'] = disk['free_for_root']
+            stats['storage_server.disk_free_for_nonroot'] = disk['free_for_nonroot']
+            stats['storage_server.disk_avail'] = disk['avail']
         except AttributeError:
-            # os.statvfs is available only on unix
-            pass
-        stats["storage_server.accepting_immutable_shares"] = int(writeable)
+            writeable = True
+        except EnvironmentError:
+            log.msg("OS call to get disk statistics failed", level=log.UNUSUAL)
+            writeable = False
+
+        if self.readonly_storage:
+            stats['storage_server.disk_avail'] = 0
+            writeable = False
+
+        stats['storage_server.accepting_immutable_shares'] = int(writeable)
         s = self.bucket_counter.get_state()
         bucket_count = s.get("last-complete-bucket-count")
         if bucket_count:
-            stats["storage_server.total_bucket_count"] = bucket_count
+            stats['storage_server.total_bucket_count'] = bucket_count
         return stats
 
-
-    def stat_disk(self, d):
-        s = os.statvfs(d)
-        # s.f_bavail: available to non-root users
-        disk_avail = s.f_frsize * s.f_bavail
-        return disk_avail
-
     def get_available_space(self):
-        # returns None if it cannot be measured (windows)
-        try:
-            disk_avail = self.stat_disk(self.storedir)
-            disk_avail -= self.reserved_space
-        except AttributeError:
-            disk_avail = None
+        """Returns available space for share storage in bytes, or None if no
+        API to get this information is available."""
+
         if self.readonly_storage:
-            disk_avail = 0
-        return disk_avail
+            return 0
+        try:
+            return self.get_disk_stats()['avail']
+        except AttributeError:
+            return None
+        except EnvironmentError:
+            log.msg("OS call to get disk statistics failed", level=log.UNUSUAL)
+            return 0
 
     def allocated_size(self):
         space = 0
@@ -232,9 +270,9 @@ class StorageServer(service.MultiService, Referenceable):
     def remote_get_version(self):
         remaining_space = self.get_available_space()
         if remaining_space is None:
-            # we're on a platform that doesn't have 'df', so make a vague
-            # guess.
+            # We're on a platform that has no API to get disk stats.
             remaining_space = 2**64
+
         version = { "http://allmydata.org/tahoe/protocols/storage/v1" :
                     { "maximum-immutable-share-size": remaining_space,
                       "tolerates-immutable-read-overrun": True,
@@ -278,6 +316,7 @@ class StorageServer(service.MultiService, Referenceable):
             # has already been written to disk, where it will show up in
             # get_available_space.
             remaining_space -= self.allocated_size()
+        # self.readonly_storage causes remaining_space <= 0
 
         # fill alreadygot with all shares that we have, not just the ones
         # they asked about: this will save them a lot of work. Add or update
@@ -288,8 +327,6 @@ class StorageServer(service.MultiService, Referenceable):
             sf = ShareFile(fn)
             sf.add_or_renew_lease(lease_info)
 
-        # self.readonly_storage causes remaining_space=0
-
         for shnum in sharenums:
             incominghome = os.path.join(self.incomingdir, si_dir, "%d" % shnum)
             finalhome = os.path.join(self.sharedir, si_dir, "%d" % shnum)
@@ -445,7 +482,7 @@ class StorageServer(service.MultiService, Referenceable):
         start = time.time()
         self.count("writev")
         si_s = si_b2a(storage_index)
-        lp = log.msg("storage: slot_writev %s" % si_s)
+        log.msg("storage: slot_writev %s" % si_s)
         si_dir = storage_index_to_dir(storage_index)
         (write_enabler, renew_secret, cancel_secret) = secrets
         # shares exist if there is a file for them
