@@ -23,6 +23,8 @@ if os.path.basename(srcdir) == 'site-packages':
     if re.search(r'python.+\..+', os.path.basename(rootdir)):
         rootdir = os.path.dirname(rootdir)
     rootdir = os.path.dirname(rootdir)
+elif os.path.basename(rootdir) == 'src':
+    rootdir = os.path.dirname(rootdir)
 
 bintahoe = os.path.join(rootdir, 'bin', 'tahoe')
 if sys.platform == "win32":
@@ -51,6 +53,8 @@ class BinTahoe(common_util.SignalMixin, unittest.TestCase, SkipMixin):
     def test_the_right_code(self):
         cwd = os.path.normcase(os.path.realpath("."))
         root_from_cwd = os.path.dirname(cwd)
+        if os.path.basename(root_from_cwd) == 'src':
+            root_from_cwd = os.path.dirname(root_from_cwd)
 
         same = (root_from_cwd == rootdir)
         if not same:
@@ -64,8 +68,12 @@ class BinTahoe(common_util.SignalMixin, unittest.TestCase, SkipMixin):
                    "(according to the source filename %r),\n"
                    "but expected to be testing the code at %r.\n"
                    % (rootdir, srcfile, root_from_cwd))
-            if (not isinstance(cwd, unicode) and
-                cwd.decode(get_filesystem_encoding(), 'replace') != os.path.normcase(os.path.realpath(os.getcwdu()))):
+
+            root_from_cwdu = os.path.normcase(os.path.normpath(os.getcwdu()))
+            if os.path.basename(root_from_cwdu) == u'src':
+                root_from_cwdu = os.path.dirname(root_from_cwdu)
+
+            if not isinstance(root_from_cwd, unicode) and root_from_cwd.decode(get_filesystem_encoding(), 'replace') != root_from_cwdu:
                 msg += ("However, this may be a false alarm because the current directory path\n"
                         "is not representable in the filesystem encoding. Please run the tests\n"
                         "from the root of the Tahoe-LAFS distribution at a non-Unicode path.")
@@ -157,7 +165,11 @@ class BinTahoe(common_util.SignalMixin, unittest.TestCase, SkipMixin):
             self.failUnlessEqual(rc_or_sig, 0, str(res))
             self.failUnless(out.startswith(allmydata.__appname__+':'), str(res))
             self.failIfIn("DeprecationWarning", out, str(res))
-            self.failUnlessEqual(err, "", str(res))
+            errlines = err.split("\n")
+            self.failIf([True for line in errlines if (line != "" and "UserWarning: Unbuilt egg for setuptools" not in line
+                                                                  and "from pkg_resources import load_entry_point" not in line)], str(res))
+            if err != "":
+                raise unittest.SkipTest("This test is known not to pass on Ubuntu Lucid; see #1235.")
         d.addCallback(_cb)
         return d
 
@@ -234,20 +246,7 @@ class CreateNode(unittest.TestCase):
         self.failUnless(os.path.exists(n3))
         self.failUnless(os.path.exists(os.path.join(n3, tac)))
 
-        # test the --multiple form
-        n4 = os.path.join(basedir, command + "-n4")
-        n5 = os.path.join(basedir, command + "-n5")
-        argv = ["--quiet", command, "--multiple", n4, n5]
-        rc, out, err = self.run_tahoe(argv)
-        self.failUnlessEqual(err, "")
-        self.failUnlessEqual(out, "")
-        self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(n4))
-        self.failUnless(os.path.exists(os.path.join(n4, tac)))
-        self.failUnless(os.path.exists(n5))
-        self.failUnless(os.path.exists(os.path.join(n5, tac)))
-
-        # make sure it rejects too many arguments without --multiple
+        # make sure it rejects too many arguments
         argv = [command, "basedir", "extraarg"]
         self.failUnlessRaises(usage.UsageError,
                               runner.runner, argv,
@@ -439,7 +438,11 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
             open(HOTLINE_FILE, "w").write("")
             self.failUnlessEqual(rc_or_sig, 0, errstr)
             self.failUnlessEqual(out, "", errstr) # If you emit noise, you fail this test.
-            self.failUnlessEqual(err, "", errstr)
+            errlines = err.split("\n")
+            self.failIf([True for line in errlines if (line != "" and "UserWarning: Unbuilt egg for setuptools" not in line
+                                                                  and "from pkg_resources import load_entry_point" not in line)], errstr)
+            if err != "":
+                raise unittest.SkipTest("This test is known not to pass on Ubuntu Lucid; see #1235.")
 
             # the parent (twistd) has exited. However, twistd writes the pid
             # from the child, not the parent, so we can't expect twistd.pid
@@ -456,7 +459,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         d.addCallback(lambda res: self.poll(_node_has_started))
 
         # now we can kill it. TODO: On a slow machine, the node might kill
-        # itself before we get a chance too, especially if spawning the
+        # itself before we get a chance to, especially if spawning the
         # 'tahoe stop' command takes a while.
         def _stop(res):
             self.failUnless(os.path.exists(TWISTD_PID_FILE), (TWISTD_PID_FILE, os.listdir(os.path.dirname(TWISTD_PID_FILE))))
